import { useRef, useMemo, useCallback } from 'react';
export default function useItemKeys(_ref) {
  var getItemId = _ref.getItemId;
  // List items are rendered with `key`s so that React doesn't
  // "reuse" `itemComponent`s in cases when `items` are changed.
  var itemKeyPrefix = useRef(); // Generates a unique `key` prefix for list item components.

  var generateItemKeyPrefix = useMemo(function () {
    var counter = 0;

    function getNextCounter() {
      if (counter === Number.MAX_SAFE_INTEGER) {
        counter = 0;
      }

      counter++;
      return counter;
    }

    return function () {
      itemKeyPrefix.current = String(getNextCounter());
    };
  }, [itemKeyPrefix]);
  useMemo(function () {
    // Generate an initial unique `key` prefix for list item components.
    generateItemKeyPrefix();
  }, []);
  var generateItemKeyPrefixIfNotUsingItemIds = useCallback(function () {
    if (!getItemId) {
      generateItemKeyPrefix();
    }
  }, [getItemId, generateItemKeyPrefix]);
  /**
   * Returns a `key` for an `item`'s element.
   * @param  {object} item — The item.
   * @param  {number} i — Item's index in `items` list.
   * @return {any}
   */

  var getItemKey = useCallback(function (item, i) {
    if (getItemId) {
      return getItemId(item);
    }

    return "".concat(itemKeyPrefix.current, ":").concat(i);
  }, [getItemId, itemKeyPrefix]);
  return {
    getItemKey: getItemKey,
    updateItemKeysForNewItems: generateItemKeyPrefixIfNotUsingItemIds
  };
}
//# sourceMappingURL=useItemKeys.js.map