function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import { useState, useRef, useCallback, useLayoutEffect } from 'react'; // Creates state management functions.

export default function _useState(_ref) {
  var initialState = _ref.initialState,
      onRender = _ref.onRender,
      itemsProperty = _ref.itemsProperty,
      USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION = _ref.USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION;

  // This is a utility state variable that is used to re-render the component.
  // It should not be used to access the current `VirtualScroller` state.
  // It's more of a "requested" `VirtualScroller` state.
  //
  // It will also be stale in cases when `USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION`
  // feature is used for setting new `items` in state.
  //
  var _useState2 = useState(initialState),
      _useState3 = _slicedToArray(_useState2, 2),
      _newState = _useState3[0],
      _setNewState = _useState3[1]; // This `state` reference is what `VirtualScroller` uses internally.
  // It's the "source of truth" on the actual `VirtualScroller` state.


  var state = useRef(initialState);
  var setState = useCallback(function (newState) {
    state.current = newState;
  }, []); // Accumulates all "pending" state updates until they have been applied.

  var nextState = useRef(initialState); // Updates the actual `VirtualScroller` state right after a requested state update
  // has been applied. Doesn't do anything at initial render.

  useLayoutEffect(function () {
    setState(_newState);
  }, [_newState]); // Calls `onRender()` right after every state update (which is a re-render),
  // and also right after the initial render.

  useLayoutEffect(function () {
    onRender();
  }, [_newState, // When using `USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION` feature,
  // there won't be a `_setNewState()` function call when `items` property changes,
  // hence the additional `itemsProperty` dependency.
  USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION ? itemsProperty : undefined]);
  return {
    getState: function getState() {
      return state.current;
    },
    getNextState: function getNextState() {
      return nextState.current;
    },
    // Requests a state update.
    //
    // State updates are incremental meaning that this function mimicks
    // the classic `React.Component`'s `this.setState()` behavior
    // when calling `this.setState()` didn't replace `state` but rather merged
    // the updated state properties over the "old" state properties.
    //
    // The reason for using pending state updates accumulation is that
    // `useState()` updates are "asynchronous" (not immediate),
    // and simply merging over `...state` would merge over potentially stale
    // property values in cases when more than a single `updateState()` call is made
    // before the state actually updates, resulting in losing some of those state updates.
    //
    // Example: the first `updateState()` call updates shown item indexes,
    // and the second `updateState()` call updates `verticalSpacing`.
    // If it was simply `updateState({ ...state, ...stateUpdate })`
    // then the second state update could overwrite the first state update,
    // resulting in incorrect items being shown/hidden.
    //
    updateState: function updateState(stateUpdate) {
      nextState.current = _objectSpread(_objectSpread({}, nextState.current), stateUpdate); // If `items` property did change, the component detects it at render time
      // and updates `VirtualScroller` items immediately by calling `.setItems()`,
      // which, in turn, immediately calls this `updateState()` function
      // with a `stateUpdate` argument that contains the new `items`,
      // so checking for `stateUpdate.items` could detect situations like that.
      //
      // When the initial `VirtualScroller` state is being set, it contains the `.items`
      // property too, but that initial setting is done using another function called
      // `setInitialState()`, so using `if (stateUpdate.items)` condition here for describing
      // just the case when `state` has been updated as a result of a `setItems()` call
      // seems to be fine.
      //

      var _newState = nextState.current;

      if (stateUpdate.items && USE_ITEMS_UPDATE_NO_SECOND_RENDER_OPTIMIZATION) {
        setState(_newState);
      } else {
        _setNewState(_newState);
      }
    }
  };
}
//# sourceMappingURL=useState.js.map